@@ -5,7 +5,7 @@ test_that("Test graph construction from lines", {
 
   graph <-  metric_graph$new(edges = list(edge1, edge2))
 
-  expect_equal(graph$V, matrix(c(0, 0, 1, 0, 1, 1),3, 2), tol = 1e-9)
+  expect_equal(sum((graph$V - matrix(c(0, 0, 1, 0, 1, 1),3, 2))^2),0, tol = 1e-9)
   expect_equal(graph$E, matrix(c(1, 2, 2, 3), 2, 2), tol = 1e-9)
 })
 
@@ -27,7 +27,7 @@ test_that("Test adding point", {
   graph$add_observations(data = dat)
   graph$observation_to_vertex()
 
-  expect_equal(graph$V, matrix(c(0, 0, 1, 0, 0, 1, 1, 0.5), 4, 2), tol=1e-9)
+  expect_equal(sum((graph$V - matrix(c(0, 0, 1, 0, 0, 1, 1, 0.5), 4, 2))^2),0, tol=1e-9)
   expect_equal(graph$E, matrix(c(1, 2, 4, 4, 3, 2), 3, 2), tol=1e-9)
 })
 
